/*
 * emu2408.c
 *
 *  Created on: May 10, 2019
 *      Author: harry
 */

#include "emu2408.h"

#include "i2c.h"
#include "string.h"

volatile static uint8_t state;
volatile static uint16_t ee_ptr;

#define SLAVE_OWN_ADDRESS	0xa0

#define S_IDLE		0
#define S_RXDTA		1
#define S_ADDR		2
#define S_DTA_SND	3
#define S_RDOUT		4
#define S_DTA_RCVD	5

void emu2408_init(void)
{
//	memcpy(eeprom, eeprom_default, 1024);
	state = S_IDLE;

// LL-Driver Bugfix
	LL_I2C_DisableOwnAddress2(I2C2);
	LL_I2C_SetOwnAddress2(I2C2, SLAVE_OWN_ADDRESS, LL_I2C_OWNADDRESS2_MASK02);
	LL_I2C_EnableOwnAddress2(I2C2);

	/* Enable Peripheral in I2C mode */
	LL_I2C_Enable(I2C2);

	/* (6) Enable I2C2 address match/error interrupts:
	 *  - Enable Address Match Interrupt
	 *  - Enable Error interrupts
	 *  - Enable Stop interrupt
	 */
	LL_I2C_EnableIT_ADDR(I2C2);
	LL_I2C_EnableIT_ERR(I2C2);
	LL_I2C_EnableIT_STOP(I2C2);

}

/*
 * Callback functions
 */

void Address_Callback(void);
void Rx_Callback(void);
void Tx_Callback(void);

__weak void emu2408_Write(uint16_t Address, uint8_t Data)
{
	UNUSED(Address);
	UNUSED(Data);
}

__weak uint8_t emu2408_Read(uint16_t Address)
{
	UNUSED(Address);
	return 0xff;
}


void Address_Callback(void)
{
	uint32_t AddrMatchCode;
	/* Verify the Address Match with the OWN Slave address */
	AddrMatchCode = LL_I2C_GetAddressMatchCode(I2C2);
	if ((AddrMatchCode & 0xf8) == SLAVE_OWN_ADDRESS)
	{
		ee_ptr = (ee_ptr & 0xff) | ((AddrMatchCode & 0x06) << 7);
		/* Verify the transfer direction, a read direction, Slave enters transmitter mode */
		if (LL_I2C_GetTransferDirection(I2C2) == LL_I2C_DIRECTION_READ)
		{
			/* Enable Transmit Interrupt */
//			LL_I2C_DisableIT_RX(I2C2);
			LL_I2C_EnableIT_TX(I2C2);
			LL_I2C_EnableIT_TC(I2C2);
			state = S_RDOUT;
		}
		else
		{
//			LL_I2C_DisableIT_TX(I2C2);
			LL_I2C_EnableIT_RX(I2C2);
			LL_I2C_EnableIT_TC(I2C2);
			state = S_ADDR;
		}
	}
}

void Rx_Callback(void)
{
	uint8_t a;

	a = LL_I2C_ReceiveData8(I2C2);
	switch (state)
	{
	case S_ADDR:
		ee_ptr = (ee_ptr & 0x300) | a;
		state = S_RXDTA;
		break;

	case S_RXDTA:
		emu2408_Write(ee_ptr++, a);
		break;

	default:
		break;
	}
}

void Tx_Callback(void)
{
	LL_I2C_TransmitData8(I2C2, emu2408_Read(ee_ptr++));
}

void TransferCompleteCallback(void)
{
	LL_I2C_DisableIT_TC(I2C2);
	LL_I2C_DisableIT_TX(I2C2);
	LL_I2C_DisableIT_RX(I2C2);

}

void STOP_Callback(void)
{
	state = S_IDLE;
	/* Clear STOP flag value in ISR register */
	LL_I2C_ClearFlag_STOP(I2C2);

	/* Check TXE flag value in ISR register */
	if (!LL_I2C_IsActiveFlag_TXE(I2C2))
	{
		/* Flush the TXDR register */
		LL_I2C_ClearFlag_TXE(I2C2);
	}

}

/**
 * @brief This function handles I2C2 event global interrupt / I2C2 wake-up interrupt through EXTI line 24.
 */
void I2C2_EV_IRQHandler(void)
{
	/* Check ADDR flag value in ISR register */
	if (LL_I2C_IsActiveFlag_ADDR(I2C2))
	{
		Address_Callback();
		/* Clear ADDR flag value in ISR register */
		LL_I2C_ClearFlag_ADDR(I2C2);

	}

	else if (LL_I2C_IsActiveFlag_RXNE(I2C2))
	{
		Rx_Callback();
	}

	/* Check NACK flag value in ISR register */
	else if (LL_I2C_IsActiveFlag_NACK(I2C2))
	{
		/* End of Transfer */
		LL_I2C_ClearFlag_NACK(I2C2);
	}
	/* Check TXIS flag value in ISR register */
	else if (LL_I2C_IsActiveFlag_TXIS(I2C2))
	{
		Tx_Callback();
	}
	/* Check STOP flag value in ISR register */
	else if (LL_I2C_IsActiveFlag_STOP(I2C2))
	{
		STOP_Callback();
	}
	/* Check TXE flag value in ISR register */
	else if (!LL_I2C_IsActiveFlag_TXE(I2C2))
	{
		/* Do nothing */
		/* This Flag will be set by hardware when the TXDR register is empty */
		/* If needed, use LL_I2C_ClearFlag_TXE() interface to flush the TXDR register  */
	}
	else
	{
		/* Call Error function */
//		Error_Callback();
	}
}

/**
 * @brief This function handles I2C2 error interrupt.
 */
void I2C2_ER_IRQHandler(void)
{
}

